@@ -1,4 +1,21 @@
-/*
+/*  Copyright 1996 Grant R. Guenther,  based on work of Itai Nahshon
+ *   http://www.torque.net/ziptool.html
+ *  Copyright 1997-2002,2007-2009 Alain Knaff.
+ *  This file is part of mtools.
+ *
+ *  Mtools is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Mtools is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with Mtools.  If not, see <http://www.gnu.org/licenses/>.
+ *
  * mzip.c
  * Iomega Zip/Jaz drive tool
  * change protection mode and eject disk
@@ -134,7 +151,7 @@ static int test_mounted ( char *dev )
 }
 
 
-static void usage(void)
+static void usage(int ret)
 {
 	fprintf(stderr, 
 		"Mtools version %s, dated %s\n", 
@@ -150,7 +167,7 @@ static void usage(void)
 		"\t-x password protected\n"
 		"\t-u unprotect till disk ejecting\n", 
 		progname);
-	exit(1);
+	exit(ret);
 }
 
 
@@ -177,7 +194,7 @@ static enum mode_t get_zip_status(int priv, int fd, void *extra_data)
 
 
 static int short_command(int priv, int fd, int cmd1, int cmd2, 
-			 int cmd3, char *data, void *extra_data)
+			 int cmd3, const char *data, void *extra_data)
 {
 	unsigned char cdb[6] = { 0, 0, 0, 0, 0, 0 };
 
@@ -186,11 +203,11 @@ static int short_command(int priv, int fd, int cmd1, int cmd2,
 	cdb[4] = cmd3;
 
 	return zip_cmd(priv, fd, cdb, 6, SCSI_IO_WRITE, 
-		       data, data ? strlen(data) : 0, extra_data);
+		       (char *) data, data ? strlen(data) : 0, extra_data);
 }
 
 
-static int iomega_command(int priv, int fd, int mode, char *data, 
+static int iomega_command(int priv, int fd, int mode, const char *data, 
 			  void *extra_data)
 {
 	return short_command(priv, fd, 
@@ -204,7 +221,7 @@ static int door_command(int priv, int fd, int cmd1, int cmd2,
 	return short_command(priv, fd, cmd1, 0, cmd2, 0, extra_data);
 }
 
-void mzip(int argc, char **argv, int type)
+void mzip(int argc, char **argv, int type UNUSEDP)
 {
 	void *extra_data;
 	int c;
@@ -223,16 +240,18 @@ void mzip(int argc, char **argv, int type)
 	enum mode_t oldMode = ZIP_RW;
 
 #define setMode(x) \
-	if(request & ZIP_MODE_CHANGE) usage(); \
+	if(request & ZIP_MODE_CHANGE) usage(1); \
 	request |= ZIP_MODE_CHANGE; \
 	newMode = x; \
 	break;
 	
 	/* get command line options */
-	while ((c = getopt(argc, argv, "i:efpqrwxu")) != EOF) {
+	if(helpFlag(argc, argv))
+		usage(0);
+	while ((c = getopt(argc, argv, "i:efpqrwxuh")) != EOF) {
 		switch (c) {
 			case 'i':
-				set_cmd_line_image(optarg, SCSI_FLAG);
+				set_cmd_line_image(optarg);
 				break;
 			case 'f':
 				if (get_real_uid()) {
@@ -259,8 +278,11 @@ void mzip(int argc, char **argv, int type)
 				setMode(ZIP_PW);
 			case 'u': /* password protected */
 				setMode(ZIP_UNLOCK_TIL_EJECT)
-					default:  /* unrecognized */
-						usage();
+			case 'h':
+				usage(0);
+			default:  /* unrecognized */
+				usage(1);
+			
 		}
 	}
 	
@@ -269,7 +291,7 @@ void mzip(int argc, char **argv, int type)
 	if (argc - optind > 1 || 
 	    (argc - optind == 1 &&
 	     (!argv[optind][0] || argv[optind][1] != ':')))
-		usage();
+		usage(1);
 	
 	drive = toupper(argc - optind == 1 ? argv[argc - 1][0] : ':');
 	
@@ -394,7 +416,8 @@ void mzip(int argc, char **argv, int type)
 	if (request & ZIP_MODE_CHANGE) {
 		int ret;
 		enum mode_t unlockMode, unlockMask;
-		char *passwd, dummy[1];
+		const char *passwd;
+		char dummy[1];
 
 		if(newMode == ZIP_UNLOCK_TIL_EJECT) {
 			unlockMode = newMode | oldMode;
