@@ -15,10 +15,10 @@
  *  along with Mtools.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-const char *mversion="4.0.17";
+const char *mversion="pre-4.0.20";
 
 /* Multiple releases on same day should be marked with (b), (cd), (d) after
  * date string below */
-const char *mdate = "June 29th, 2011";
+const char *mdate = "November 11th, 2018";
 
-const char *mformat_banner = "MTOO4017";
+const char *mformat_banner = "MTOO4020";
